@@ -24,7 +24,6 @@
 #include <stdlib.h>
 #endif
 
-#include <exo/exo.h>
 #include <dbus/dbus-glib.h>
 #include <libxfce4util/libxfce4util.h>
 #include <common/panel-private.h>
@@ -70,8 +69,9 @@ panel_dbus_client_get_proxy (GError **error)
 
 
 gboolean
-panel_dbus_client_display_preferences_dialog (guint    active,
-                                              GError **error)
+panel_dbus_client_display_preferences_dialog (guint         active,
+                                              const gchar  *socket_id,
+                                              GError      **error)
 {
   gboolean    result;
   DBusGProxy *dbus_proxy;
@@ -83,7 +83,8 @@ panel_dbus_client_display_preferences_dialog (guint    active,
     return FALSE;
 
   result = _panel_dbus_client_display_preferences_dialog (dbus_proxy,
-                                                          active, error);
+                                                          active, socket_id,
+                                                          error);
 
   g_object_unref (G_OBJECT (dbus_proxy));
 
@@ -182,6 +183,7 @@ panel_dbus_client_gtype_from_string (const gchar *str)
 
 gboolean
 panel_dbus_client_plugin_event (const gchar  *plugin_event,
+                                gboolean     *return_succeed,
                                 GError      **error)
 {
   DBusGProxy  *dbus_proxy;
@@ -201,8 +203,8 @@ panel_dbus_client_plugin_event (const gchar  *plugin_event,
   n_tokens = g_strv_length (tokens);
 
   if (!(n_tokens == 2 || n_tokens == N_TOKENS)
-      || exo_str_is_empty (tokens[PLUGIN_NAME])
-      || exo_str_is_empty (tokens[NAME]))
+      || panel_str_is_empty (tokens[PLUGIN_NAME])
+      || panel_str_is_empty (tokens[NAME]))
     {
       g_set_error_literal (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE,
                            _("Invalid plugin event syntax specified. "
@@ -256,6 +258,7 @@ panel_dbus_client_plugin_event (const gchar  *plugin_event,
                                             tokens[PLUGIN_NAME],
                                             tokens[NAME],
                                             &value,
+                                            return_succeed,
                                             error);
   g_value_unset (&value);
 
