@@ -27,7 +27,6 @@
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-#include <exo/exo.h>
 #include <common/panel-private.h>
 #include <common/panel-dbus.h>
 #include <libxfce4util/libxfce4util.h>
@@ -256,7 +255,7 @@ panel_dbus_service_add_new_item (PanelDBusService  *service,
 
   application = panel_application_get ();
 
-  if (arguments != NULL && exo_str_is_empty (*arguments))
+  if (arguments != NULL && panel_str_is_empty (*arguments))
     arguments = NULL;
 
   /* add new plugin (with or without arguments) */
