@@ -25,7 +25,6 @@
 #include <libxfce4panel/libxfce4panel.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
-#include <exo/exo.h>
 #include <dbus/dbus-glib.h>
 
 #include <common/panel-private.h>
@@ -216,7 +215,6 @@ XFCE_PANEL_DEFINE_PLUGIN (ActionsPlugin, actions_plugin)
 
 
 
-static GtkIconSize menu_icon_size = GTK_ICON_SIZE_INVALID;
 static GQuark      action_quark = 0;
 
 
@@ -243,7 +241,7 @@ actions_plugin_class_init (ActionsPluginClass *klass)
                                    g_param_spec_boxed ("items",
                                                        NULL, NULL,
                                                        PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_APPEARANCE,
@@ -252,27 +250,21 @@ actions_plugin_class_init (ActionsPluginClass *klass)
                                                       APPEARANCE_TYPE_BUTTONS,
                                                       APPEARANCE_TYPE_MENU,
                                                       APPEARANCE_TYPE_MENU,
-                                                      EXO_PARAM_READWRITE));
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_INVERT_ORIENTATION,
                                    g_param_spec_boolean ("invert-orientation",
                                                          NULL, NULL,
                                                          FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ASK_CONFIRMATION,
                                    g_param_spec_boolean ("ask-confirmation",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
-
-  menu_icon_size = gtk_icon_size_from_name ("panel-actions-menu");
-  if (menu_icon_size == GTK_ICON_SIZE_INVALID)
-    menu_icon_size = gtk_icon_size_register ("panel-actions-menu",
-                                             DEFAULT_ICON_SIZE,
-                                             DEFAULT_ICON_SIZE);
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   action_quark = g_quark_from_string ("panel-action-quark");
 }
@@ -409,18 +401,6 @@ actions_plugin_free_data (XfcePanelPlugin *panel_plugin)
 
 
 
-static void
-actions_plugin_size_changed_child (GtkWidget *child,
-                                   gpointer   data)
-{
-  gint size = GPOINTER_TO_INT (data);
-
-  if (!GTK_IS_SEPARATOR (child))
-    gtk_widget_set_size_request (child, size, size);
-}
-
-
-
 static gboolean
 actions_plugin_size_changed (XfcePanelPlugin *panel_plugin,
                              gint             size)
@@ -428,9 +408,9 @@ actions_plugin_size_changed (XfcePanelPlugin *panel_plugin,
   ActionsPlugin *plugin = XFCE_ACTIONS_PLUGIN (panel_plugin);
   GtkWidget     *box;
   GList         *children, *li;
-  gint           n_children;
-  gint           child_size;
   gint           max_size;
+  GtkImage      *icon;
+  gint           icon_size;
 
   if (plugin->type == APPEARANCE_TYPE_BUTTONS)
     {
@@ -438,29 +418,17 @@ actions_plugin_size_changed (XfcePanelPlugin *panel_plugin,
       box = gtk_bin_get_child (GTK_BIN (plugin));
       if (box != NULL)
         {
-          if (plugin->invert_orientation !=
-              (xfce_panel_plugin_get_mode (panel_plugin) == XFCE_PANEL_PLUGIN_MODE_DESKBAR))
-            {
-              children = gtk_container_get_children (GTK_CONTAINER (box));
-              if (G_UNLIKELY (children == NULL))
-                return TRUE;
-              n_children = g_list_length (children);
+          children = gtk_container_get_children (GTK_CONTAINER (box));
+          if (G_UNLIKELY (children == NULL))
+            return TRUE;
 
-              for (li = children; li != NULL; li = li->next)
-                {
-                  child_size = MIN (size / n_children, max_size);
-                  size -= child_size;
-                  n_children--;
-
-                  gtk_widget_set_size_request (GTK_WIDGET (li->data),
-                                               child_size, child_size);
-                }
-            }
-          else
+          for (li = children; li != NULL; li = li->next)
             {
-              gtk_container_foreach (GTK_CONTAINER (box),
-                  actions_plugin_size_changed_child,
-                  GINT_TO_POINTER (max_size));
+              gtk_widget_set_size_request (GTK_WIDGET (li->data),
+                                           max_size, max_size);
+              icon = GTK_IMAGE (gtk_bin_get_child (GTK_BIN (li->data)));
+              icon_size = xfce_panel_plugin_get_icon_size (panel_plugin);
+              gtk_image_set_pixel_size (GTK_IMAGE (icon), icon_size);
             }
         }
     }
@@ -596,18 +564,22 @@ actions_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
     return;
 
   combo = gtk_builder_get_object (builder, "combo-mode");
-  exo_mutual_binding_new (G_OBJECT (plugin), "appearance",
-                          G_OBJECT (combo), "active");
+  g_object_bind_property (G_OBJECT (plugin), "appearance",
+                          G_OBJECT (combo), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   object = gtk_builder_get_object (builder, "invert-orientation");
-  exo_mutual_binding_new (G_OBJECT (plugin), "invert-orientation",
-                          G_OBJECT (object), "active");
-  exo_binding_new_with_negation (G_OBJECT (combo), "active",
-                                 G_OBJECT (object), "sensitive");
+  g_object_bind_property (G_OBJECT (plugin), "invert-orientation",
+                          G_OBJECT (object), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
+  g_object_bind_property (G_OBJECT (combo), "active",
+                          G_OBJECT (object), "sensitive",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL | G_BINDING_INVERT_BOOLEAN);
 
   object = gtk_builder_get_object (builder, "confirmation-dialog");
-  exo_mutual_binding_new (G_OBJECT (plugin), "ask-confirmation",
-                          G_OBJECT (object), "active");
+  g_object_bind_property (G_OBJECT (plugin), "ask-confirmation",
+                          G_OBJECT (object), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   store = gtk_builder_get_object (builder, "items-store");
   panel_return_if_fail (GTK_IS_LIST_STORE (store));
@@ -626,7 +598,7 @@ actions_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
       /* get the value and check if it is within range */
       val = g_ptr_array_index (plugin->items, i);
       name = g_value_get_string (val);
-      if (exo_str_is_empty (name))
+      if (panel_str_is_empty (name))
         continue;
 
       /* find the entry in the available actions */
@@ -886,10 +858,10 @@ actions_plugin_actions_allowed (void)
 
           if (actions_plugin_action_dbus_can (proxy, "CanRestart"))
             PANEL_SET_FLAG (allow_mask, ACTION_TYPE_RESTART);
-        
+
           if (actions_plugin_action_dbus_can (proxy, "CanSuspend"))
             PANEL_SET_FLAG (allow_mask, ACTION_TYPE_SUSPEND);
-          
+
           if (actions_plugin_action_dbus_can (proxy, "CanHibernate"))
             PANEL_SET_FLAG (allow_mask, ACTION_TYPE_HIBERNATE);
 
@@ -1007,10 +979,7 @@ actions_plugin_action_button (ActionsPlugin  *plugin,
 
   if (entry->type == ACTION_TYPE_SEPARATOR)
     {
-      if (orientation == GTK_ORIENTATION_HORIZONTAL)
-        widget = gtk_vseparator_new ();
-      else
-        widget = gtk_hseparator_new ();
+      widget = gtk_separator_new (orientation);
     }
   else
     {
@@ -1021,7 +990,7 @@ actions_plugin_action_button (ActionsPlugin  *plugin,
       g_signal_connect (G_OBJECT (widget), "clicked",
           G_CALLBACK (actions_plugin_action_activate), plugin);
 
-      image = xfce_panel_image_new_from_source (entry->icon_name);
+      image = gtk_image_new_from_icon_name (entry->icon_name, GTK_ICON_SIZE_BUTTON);
       gtk_container_add (GTK_CONTAINER (widget), image);
       gtk_widget_show (image);
     }
@@ -1036,7 +1005,6 @@ actions_plugin_action_button (ActionsPlugin  *plugin,
 static GtkWidget *
 actions_plugin_action_menu_item (ActionsPlugin *plugin,
                                  const gchar   *name,
-                                 gint           size,
                                  ActionType    *type)
 {
   GtkWidget   *mi;
@@ -1054,18 +1022,18 @@ actions_plugin_action_menu_item (ActionsPlugin *plugin,
   if (entry->type == ACTION_TYPE_SEPARATOR)
     return gtk_separator_menu_item_new ();
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   mi = gtk_image_menu_item_new_with_mnemonic (_(entry->mnemonic));
+G_GNUC_END_IGNORE_DEPRECATIONS
   g_object_set_qdata (G_OBJECT (mi), action_quark, entry);
   g_signal_connect (G_OBJECT (mi), "activate",
       G_CALLBACK (actions_plugin_action_activate), plugin);
 
-  if (size > 0)
-    {
-      image = xfce_panel_image_new_from_source (entry->icon_name);
-      xfce_panel_image_set_size (XFCE_PANEL_IMAGE (image), size);
-      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-      gtk_widget_show (image);
-    }
+  image = gtk_image_new_from_icon_name (entry->icon_name, GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
+  gtk_widget_show (image);
 
   return mi;
 }
@@ -1111,7 +1079,7 @@ actions_plugin_pack_idle (gpointer data)
 
       if (plugin->invert_orientation)
         orientation = !orientation;
-      box = xfce_hvbox_new (orientation, FALSE, 0);
+      box = gtk_box_new (orientation, 0);
       gtk_container_add (GTK_CONTAINER (plugin), box);
       gtk_widget_show (box);
 
@@ -1145,11 +1113,11 @@ actions_plugin_pack_idle (gpointer data)
     {
       /* get a decent username, not the glib defaults */
       username = g_get_real_name ();
-      if (exo_str_is_empty (username)
+      if (panel_str_is_empty (username)
           || strcmp (username, "Unknown") == 0)
         {
           username = g_get_user_name ();
-          if (exo_str_is_empty (username)
+          if (panel_str_is_empty (username)
               || strcmp (username, "somebody") == 0)
             username = _("John Doe");
         }
@@ -1168,6 +1136,9 @@ actions_plugin_pack_idle (gpointer data)
       mode = xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin));
       gtk_label_set_angle (GTK_LABEL (label),
           (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? 270 : 0);
+      gtk_label_set_ellipsize (GTK_LABEL (label),
+                               (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR) ?
+                               PANGO_ELLIPSIZE_END : PANGO_ELLIPSIZE_NONE);
       gtk_widget_show (label);
     }
 
@@ -1254,7 +1225,6 @@ actions_plugin_menu (GtkWidget     *button,
   const GValue *val;
   const gchar  *name;
   GtkWidget    *mi;
-  gint          w, h, size;
   ActionType    type;
   ActionType    allowed_types;
 
@@ -1272,10 +1242,6 @@ actions_plugin_menu (GtkWidget     *button,
           G_CALLBACK (actions_plugin_menu_deactivate), button);
       g_object_add_weak_pointer (G_OBJECT (plugin->menu), (gpointer) &plugin->menu);
 
-      size = DEFAULT_ICON_SIZE;
-      if (gtk_icon_size_lookup (menu_icon_size, &w, &h))
-        size = MIN (w, h);
-
       allowed_types = actions_plugin_actions_allowed ();
 
       for (i = 0; i < plugin->items->len; i++)
@@ -1285,7 +1251,7 @@ actions_plugin_menu (GtkWidget     *button,
           if (name == NULL || *name != '+')
             continue;
 
-          mi = actions_plugin_action_menu_item (plugin, name + 1, size, &type);
+          mi = actions_plugin_action_menu_item (plugin, name + 1, &type);
           if (mi != NULL)
             {
               gtk_menu_shell_append (GTK_MENU_SHELL (plugin->menu), mi);
