@@ -18,7 +18,6 @@
 
 
 #include <glib.h>
-#include <exo/exo.h>
 
 #include "clock-time.h"
 #include "clock.h"
@@ -93,7 +92,7 @@ clock_time_class_init (ClockTimeClass *klass)
                                    g_param_spec_string ("timezone",
                                                         NULL, NULL,
                                                         DEFAULT_TIMEZONE,
-                                                        EXO_PARAM_READWRITE));
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   clock_time_signals[TIME_CHANGED] =
     g_signal_new (g_intern_static_string ("time-changed"),
@@ -107,10 +106,10 @@ clock_time_class_init (ClockTimeClass *klass)
 
 
 static void
-clock_time_init (ClockTime *clock_time)
+clock_time_init (ClockTime *time)
 {
-  clock_time->timezone_name = g_strdup (DEFAULT_TIMEZONE);
-  clock_time->timezone = g_time_zone_new_local ();
+  time->timezone_name = g_strdup (DEFAULT_TIMEZONE);
+  time->timezone = NULL;
 }
 
 
@@ -118,11 +117,12 @@ clock_time_init (ClockTime *clock_time)
 static void
 clock_time_finalize (GObject *object)
 {
-  ClockTime *clock_time = XFCE_CLOCK_TIME (object);
+  ClockTime *time = XFCE_CLOCK_TIME (object);
 
-  g_free (clock_time->timezone_name);
+  g_free (time->timezone_name);
 
-  g_time_zone_unref (clock_time->timezone);
+  if (time->timezone != NULL)
+    g_time_zone_unref (time->timezone);
 
   G_OBJECT_CLASS (clock_time_parent_class)->finalize (object);
 }
@@ -135,12 +135,12 @@ clock_time_get_property (GObject    *object,
                          GValue     *value,
                          GParamSpec *pspec)
 {
-  ClockTime *clock_time = XFCE_CLOCK_TIME (object);
+  ClockTime *time = XFCE_CLOCK_TIME (object);
 
   switch (prop_id)
     {
     case PROP_TIMEZONE:
-      g_value_set_string (value, clock_time->timezone_name);
+      g_value_set_string (value, time->timezone_name);
       break;
 
     default:
@@ -157,29 +157,30 @@ clock_time_set_property (GObject      *object,
                          const GValue *value,
                          GParamSpec   *pspec)
 {
-  ClockTime     *clock_time = XFCE_CLOCK_TIME (object);
+  ClockTime     *time = XFCE_CLOCK_TIME (object);
   const gchar   *str_value;
 
   switch (prop_id)
     {
     case PROP_TIMEZONE:
       str_value = g_value_get_string (value);
-      if (g_strcmp0 (clock_time->timezone_name, str_value) != 0)
+      if (g_strcmp0 (time->timezone_name, str_value) != 0)
         {
-          g_free (clock_time->timezone_name);
-          g_time_zone_unref (clock_time->timezone);
+          g_free (time->timezone_name);
+          if (time->timezone != NULL)
+            g_time_zone_unref (time->timezone);
           if (str_value == NULL || g_strcmp0 (str_value, "") == 0)
             {
-              clock_time->timezone_name = g_strdup (DEFAULT_TIMEZONE);
-              clock_time->timezone = g_time_zone_new_local ();
+              time->timezone_name = g_strdup (DEFAULT_TIMEZONE);
+              time->timezone = NULL;
             }
           else
             {
-              clock_time->timezone_name = g_strdup (str_value);
-              clock_time->timezone = g_time_zone_new (str_value);
+              time->timezone_name = g_strdup (str_value);
+              time->timezone = g_time_zone_new (str_value);
             }
 
-          g_signal_emit (G_OBJECT (clock_time), clock_time_signals[TIME_CHANGED], 0);
+          g_signal_emit (G_OBJECT (time), clock_time_signals[TIME_CHANGED], 0);
         }
       break;
 
@@ -192,14 +193,14 @@ clock_time_set_property (GObject      *object,
 
 
 GDateTime *
-clock_time_get_time (ClockTime *clock_time)
+clock_time_get_time (ClockTime *time)
 {
   GDateTime *date_time;
 
-  panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (clock_time), NULL);
+  panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (time), NULL);
 
-  if (clock_time->timezone != NULL)
-    date_time = g_date_time_new_now (clock_time->timezone);
+  if (time->timezone != NULL)
+    date_time = g_date_time_new_now (time->timezone);
   else
     date_time = g_date_time_new_now_local ();
 
@@ -209,15 +210,15 @@ clock_time_get_time (ClockTime *clock_time)
 
 
 gchar *
-clock_time_strdup_strftime (ClockTime       *clock_time,
+clock_time_strdup_strftime (ClockTime       *time,
                             const gchar     *format)
 {
   GDateTime *date_time;
   gchar     *str;
 
-  panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (clock_time), NULL);
+  panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (time), NULL);
 
-  date_time = clock_time_get_time (clock_time);
+  date_time = clock_time_get_time (time);
   str = g_date_time_format (date_time, format);
 
   g_date_time_unref (date_time);
@@ -232,7 +233,7 @@ clock_time_interval_from_format (const gchar *format)
 {
   const gchar *p;
 
-  if (G_UNLIKELY (exo_str_is_empty (format)))
+  if (G_UNLIKELY (panel_str_is_empty (format)))
       return CLOCK_INTERVAL_MINUTE;
 
   for (p = format; *p != '\0'; ++p)
@@ -262,7 +263,7 @@ static gboolean
 clock_time_timeout_running (gpointer user_data)
 {
   ClockTimeTimeout *timeout = user_data;
-  GDateTime        *date_time;
+  GDateTime        *time;
 
   g_signal_emit (G_OBJECT (timeout->time), clock_time_signals[TIME_CHANGED], 0);
 
@@ -270,8 +271,8 @@ clock_time_timeout_running (gpointer user_data)
   if (timeout->interval == CLOCK_INTERVAL_MINUTE)
     {
       /* sync again when we don't run on time */
-      date_time = clock_time_get_time (timeout->time);
-      timeout->restart = (g_date_time_get_second (date_time) != 0);
+      time = clock_time_get_time (timeout->time);
+      timeout->restart = (g_date_time_get_second (time) != 0);
     }
 
   return !timeout->restart;
@@ -312,13 +313,13 @@ clock_time_timeout_sync (gpointer user_data)
 
 ClockTimeTimeout *
 clock_time_timeout_new (guint       interval,
-                        ClockTime  *clock_time,
+                        ClockTime  *time,
                         GCallback   c_handler,
                         gpointer    gobject)
 {
   ClockTimeTimeout *timeout;
 
-  panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (clock_time), NULL);
+  panel_return_val_if_fail (XFCE_IS_CLOCK_TIME (time), NULL);
 
   panel_return_val_if_fail (interval > 0, NULL);
 
@@ -326,10 +327,10 @@ clock_time_timeout_new (guint       interval,
   timeout->interval = 0;
   timeout->timeout_id = 0;
   timeout->restart = FALSE;
-  timeout->time = clock_time;
+  timeout->time = time;
 
   timeout->time_changed_id =
-    g_signal_connect_swapped (G_OBJECT (clock_time), "time-changed",
+    g_signal_connect_swapped (G_OBJECT (time), "time-changed",
                               c_handler, gobject);
 
   g_object_ref (G_OBJECT (timeout->time));
@@ -345,7 +346,7 @@ void
 clock_time_timeout_set_interval (ClockTimeTimeout *timeout,
                                  guint             interval)
 {
-  GDateTime *date_time;
+  GDateTime *time;
   guint      next_interval;
   gboolean   restart;
 
@@ -372,8 +373,8 @@ clock_time_timeout_set_interval (ClockTimeTimeout *timeout,
   /* get the seconds to the next internal */
   if (interval == CLOCK_INTERVAL_MINUTE)
     {
-      date_time = clock_time_get_time (timeout->time);
-      next_interval = 60 - g_date_time_get_second (date_time);
+      time = clock_time_get_time (timeout->time);
+      next_interval = 60 - g_date_time_get_second (time);
     }
   else
     {
