@@ -25,7 +25,6 @@
 #endif
 
 #include <gio/gio.h>
-#include <exo/exo.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <garcon/garcon.h>
@@ -40,8 +39,6 @@
 #include "launcher-dialog.h"
 
 #define ARROW_BUTTON_SIZE              (12)
-#define TOOLTIP_ICON_SIZE              (32)
-#define MENU_ICON_SIZE                 (32)
 #define MENU_POPUP_DELAY               (225)
 #define NO_ARROW_INSIDE_BUTTON(plugin) ((plugin)->arrow_position != LAUNCHER_ARROW_INTERNAL \
                                         || LIST_HAS_ONE_OR_NO_ENTRIES ((plugin)->items))
@@ -135,9 +132,9 @@ static void               launcher_plugin_button_drag_leave             (GtkWidg
                                                                          GdkDragContext       *context,
                                                                          guint                 drag_time,
                                                                          LauncherPlugin       *plugin);
-static gboolean           launcher_plugin_button_expose_event           (GtkWidget            *widget,
-                                                                         GdkEventExpose       *event,
-                                                                         LauncherPlugin       *launcher);
+static gboolean           launcher_plugin_button_draw                   (GtkWidget            *widget,
+                                                                         cairo_t              *cr,
+                                                                         LauncherPlugin       *plugin);
 static void               launcher_plugin_arrow_visibility              (LauncherPlugin       *plugin);
 static gboolean           launcher_plugin_arrow_press_event             (GtkWidget            *button,
                                                                          GdkEventButton       *event,
@@ -239,8 +236,6 @@ XFCE_PANEL_DEFINE_PLUGIN_RESIDENT (LauncherPlugin, launcher_plugin)
 /* quark to attach the plugin to menu items */
 static GQuark      launcher_plugin_quark = 0;
 static guint       launcher_signals[LAST_SIGNAL];
-static GtkIconSize launcher_menu_icon_size = GTK_ICON_SIZE_INVALID;
-static GtkIconSize launcher_tooltip_icon_size = GTK_ICON_SIZE_INVALID;
 
 
 
@@ -280,28 +275,28 @@ launcher_plugin_class_init (LauncherPluginClass *klass)
                                    g_param_spec_boxed ("items",
                                                        NULL, NULL,
                                                        PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_DISABLE_TOOLTIPS,
                                    g_param_spec_boolean ("disable-tooltips",
                                                          NULL, NULL,
                                                          FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_MOVE_FIRST,
                                    g_param_spec_boolean ("move-first",
                                                          NULL, NULL,
                                                          FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_LABEL,
                                    g_param_spec_boolean ("show-label",
                                                          NULL, NULL,
                                                          FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ARROW_POSITION,
@@ -310,7 +305,7 @@ launcher_plugin_class_init (LauncherPluginClass *klass)
                                                       LAUNCHER_ARROW_DEFAULT,
                                                       LAUNCHER_ARROW_INTERNAL,
                                                       LAUNCHER_ARROW_DEFAULT,
-                                                      EXO_PARAM_READWRITE));
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   launcher_signals[ITEMS_CHANGED] =
     g_signal_new (g_intern_static_string ("items-changed"),
@@ -322,18 +317,6 @@ launcher_plugin_class_init (LauncherPluginClass *klass)
 
   /* initialize the quark */
   launcher_plugin_quark = g_quark_from_static_string ("xfce-launcher-plugin");
-
-  launcher_menu_icon_size = gtk_icon_size_from_name ("panel-launcher-menu");
-  if (launcher_menu_icon_size == GTK_ICON_SIZE_INVALID)
-    launcher_menu_icon_size = gtk_icon_size_register ("panel-launcher-menu",
-                                                      MENU_ICON_SIZE,
-                                                      MENU_ICON_SIZE);
-
-  launcher_tooltip_icon_size = gtk_icon_size_from_name ("panel-launcher-tooltip");
-  if (launcher_tooltip_icon_size == GTK_ICON_SIZE_INVALID)
-    launcher_tooltip_icon_size = gtk_icon_size_register ("panel-launcher-tooltip",
-                                                         TOOLTIP_ICON_SIZE,
-                                                         TOOLTIP_ICON_SIZE);
 }
 
 
@@ -360,7 +343,7 @@ launcher_plugin_init (LauncherPlugin *plugin)
       G_CALLBACK (launcher_plugin_icon_theme_changed), plugin);
 
   /* create the panel widgets */
-  plugin->box = xfce_hvbox_new (GTK_ORIENTATION_HORIZONTAL, FALSE, 0);
+  plugin->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_container_add (GTK_CONTAINER (plugin), plugin->box);
 
   plugin->button = xfce_panel_create_button ();
@@ -382,10 +365,10 @@ launcher_plugin_init (LauncherPlugin *plugin)
       G_CALLBACK (launcher_plugin_button_drag_drop), plugin);
   g_signal_connect (G_OBJECT (plugin->button), "drag-leave",
       G_CALLBACK (launcher_plugin_button_drag_leave), plugin);
-  g_signal_connect_after (G_OBJECT (plugin->button), "expose-event",
-      G_CALLBACK (launcher_plugin_button_expose_event), plugin);
+  g_signal_connect_after (G_OBJECT (plugin->button), "draw",
+      G_CALLBACK (launcher_plugin_button_draw), plugin);
 
-  plugin->child = xfce_panel_image_new ();
+  plugin->child = gtk_image_new ();
   gtk_container_add (GTK_CONTAINER (plugin->button), plugin->child);
 
   plugin->arrow = xfce_arrow_button_new (GTK_ARROW_UP);
@@ -551,6 +534,27 @@ err1:
 }
 
 
+static gboolean
+_exo_str_looks_like_an_uri (const gchar *str)
+{
+  const gchar *s = str;
+
+  if (G_UNLIKELY (str == NULL))
+    return FALSE;
+
+  /* <scheme> starts with an alpha character */
+  if (g_ascii_isalpha (*s))
+    {
+      /* <scheme> continues with (alpha | digit | "+" | "-" | ".")* */
+      for (++s; g_ascii_isalnum (*s) || *s == '+' || *s == '-' || *s == '.'; ++s);
+
+      /* <scheme> must be followed by ":" */
+      return (*s == ':' && *(s+1) == '/');
+    }
+
+  return FALSE;
+}
+
 
 static GarconMenuItem *
 launcher_plugin_item_load (LauncherPlugin *plugin,
@@ -568,7 +572,7 @@ launcher_plugin_item_load (LauncherPlugin *plugin,
   panel_return_val_if_fail (str != NULL, NULL);
   panel_return_val_if_fail (G_IS_FILE (plugin->config_directory), NULL);
 
-  if (G_UNLIKELY (g_path_is_absolute (str) || exo_str_looks_like_an_uri (str)))
+  if (G_UNLIKELY (g_path_is_absolute (str) || _exo_str_looks_like_an_uri (str)))
     {
       src_file = g_file_new_for_commandline_arg (str);
       if (g_file_has_prefix (src_file, plugin->config_directory))
@@ -855,7 +859,7 @@ launcher_plugin_set_property (GObject      *object,
       if (G_UNLIKELY (plugin->show_label))
         plugin->child = gtk_label_new (NULL);
       else
-        plugin->child = xfce_panel_image_new ();
+        plugin->child = gtk_image_new ();
       gtk_container_add (GTK_CONTAINER (plugin->button), plugin->child);
       gtk_widget_show (plugin->child);
 
@@ -1156,16 +1160,16 @@ launcher_plugin_remote_event (XfcePanelPlugin *panel_plugin,
 
   panel_return_val_if_fail (value == NULL || G_IS_VALUE (value), FALSE);
 
-  if (exo_str_is_equal (name, "popup")
+  if (g_strcmp0 (name, "popup") == 0
       && LIST_HAS_TWO_OR_MORE_ENTRIES (plugin->items)
-      && (plugin->menu == NULL || !GTK_WIDGET_VISIBLE (plugin->menu)))
+      && (plugin->menu == NULL || !gtk_widget_get_visible (plugin->menu)))
     {
       launcher_plugin_menu_popup (plugin);
 
       return TRUE;
     }
 
-  if (exo_str_is_equal (name, "disable-tooltips")
+  if (g_strcmp0 (name, "disable-tooltips") == 0
       && value != NULL
       && G_VALUE_HOLDS_BOOLEAN (value))
     {
@@ -1204,7 +1208,7 @@ launcher_plugin_save_delayed (LauncherPlugin *plugin)
   if (plugin->save_timeout_id != 0)
     g_source_remove (plugin->save_timeout_id);
 
-  plugin->save_timeout_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, 1,
+  plugin->save_timeout_id = gdk_threads_add_timeout_seconds_full (G_PRIORITY_LOW, 1,
       launcher_plugin_save_delayed_timeout, plugin,
       launcher_plugin_save_delayed_timeout_destroyed);
 }
@@ -1238,17 +1242,18 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
 {
   LauncherPlugin    *plugin = XFCE_LAUNCHER_PLUGIN (panel_plugin);
   gint               p_width, p_height;
-  gint               a_width, a_height;
+  gint               a_size;
+  gint               icon_size;
   gboolean           horizontal;
   LauncherArrowType  arrow_position;
 
   /* initialize the plugin size */
   size /= xfce_panel_plugin_get_nrows (panel_plugin);
   p_width = p_height = size;
-  a_width = a_height = -1;
+  icon_size = xfce_panel_plugin_get_icon_size (panel_plugin);
 
   /* add the arrow size */
-  if (GTK_WIDGET_VISIBLE (plugin->arrow))
+  if (gtk_widget_get_visible (plugin->arrow))
     {
       /* if the panel is horizontal */
       horizontal = !!(xfce_panel_plugin_get_orientation (panel_plugin) ==
@@ -1261,20 +1266,20 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
         {
         case LAUNCHER_ARROW_NORTH:
         case LAUNCHER_ARROW_SOUTH:
-          a_height = ARROW_BUTTON_SIZE;
-          if (horizontal)
-            p_width -= ARROW_BUTTON_SIZE;
-          else
-            p_height += ARROW_BUTTON_SIZE;
+          if (!horizontal)
+            {
+              gtk_widget_get_preferred_height (plugin->arrow, NULL, &a_size);
+              p_height += a_size;
+            }
           break;
 
         case LAUNCHER_ARROW_EAST:
         case LAUNCHER_ARROW_WEST:
-          a_width = ARROW_BUTTON_SIZE;
           if (horizontal)
-            p_width += ARROW_BUTTON_SIZE;
-          else
-            p_height -= ARROW_BUTTON_SIZE;
+            {
+              gtk_widget_get_preferred_width (plugin->arrow, NULL, &a_size);
+              p_width += a_size;
+            }
           break;
 
         default:
@@ -1282,16 +1287,17 @@ launcher_plugin_size_changed (XfcePanelPlugin *panel_plugin,
           panel_assert_not_reached ();
           break;
         }
-
-      /* set the arrow size */
-      gtk_widget_set_size_request (plugin->arrow, a_width, a_height);
     }
 
   /* set the panel plugin size */
   if (plugin->show_label)
     gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), -1, -1);
-  else
+  else {
     gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), p_width, p_height);
+    /* set the panel plugin icon size */
+    gtk_image_set_pixel_size (GTK_IMAGE (plugin->child), icon_size);
+  }
+
 
   return TRUE;
 }
@@ -1373,7 +1379,7 @@ launcher_plugin_pack_widgets (LauncherPlugin *plugin)
   panel_return_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin));
 
   /* leave when the arrow button is not visible */
-  if (!GTK_WIDGET_VISIBLE (plugin->arrow)
+  if (!gtk_widget_get_visible (plugin->arrow)
       || plugin->arrow_position == LAUNCHER_ARROW_INTERNAL)
     return;
 
@@ -1384,8 +1390,8 @@ launcher_plugin_pack_widgets (LauncherPlugin *plugin)
   gtk_box_reorder_child (GTK_BOX (plugin->box), plugin->arrow,
       (pos == LAUNCHER_ARROW_WEST || pos == LAUNCHER_ARROW_NORTH) ? 0 : -1);
 
-  /* set the orientation of the hvbox */
-  xfce_hvbox_set_orientation (XFCE_HVBOX (plugin->box),
+  /* set the orientation */
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (plugin->box),
       !!(pos == LAUNCHER_ARROW_WEST || pos == LAUNCHER_ARROW_EAST) ?
           GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
 }
@@ -1397,28 +1403,23 @@ launcher_plugin_tooltip_pixbuf (GdkScreen   *screen,
                                 const gchar *icon_name)
 {
   GtkIconTheme *theme;
-  gint          w, h, size;
 
   panel_return_val_if_fail (screen == NULL || GDK_IS_SCREEN (screen), NULL);
 
-  if (exo_str_is_empty (icon_name))
+  if (panel_str_is_empty (icon_name))
     return NULL;
 
-  if (gtk_icon_size_lookup (launcher_tooltip_icon_size, &w, &h))
-    size = MIN (w, h);
-  else
-    size = TOOLTIP_ICON_SIZE;
-
   /* load directly from a file */
   if (G_UNLIKELY (g_path_is_absolute (icon_name)))
-    return exo_gdk_pixbuf_new_from_file_at_max_size (icon_name, size, size, TRUE, NULL);
+    return gdk_pixbuf_new_from_file_at_scale (icon_name, GTK_ICON_SIZE_DND, GTK_ICON_SIZE_DND,
+                                              TRUE, NULL);
 
   if (G_LIKELY (screen != NULL))
     theme = gtk_icon_theme_get_for_screen (screen);
   else
     theme = gtk_icon_theme_get_default ();
 
-  return gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
+  return gtk_icon_theme_load_icon (theme, icon_name, GTK_ICON_SIZE_DND, 0, NULL);
 }
 
 
@@ -1518,7 +1519,7 @@ launcher_plugin_menu_item_drag_data_received (GtkWidget          *widget,
     }
 
   /* hide the menu */
-  gtk_widget_hide (GTK_MENU (plugin->menu)->toplevel);
+  gtk_widget_hide (gtk_widget_get_toplevel (plugin->menu));
   gtk_widget_hide (plugin->menu);
 
   /* inactivate the toggle button */
@@ -1536,10 +1537,9 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
   GtkArrowType    arrow_type;
   guint           n;
   GarconMenuItem *item;
-  GtkWidget      *mi, *image;
+  GtkWidget      *mi, *box, *label, *image;
   const gchar    *name, *icon_name;
   GSList         *li;
-  gint            w, h, size;
 
   panel_return_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin));
   panel_return_if_fail (plugin->menu == NULL);
@@ -1553,12 +1553,6 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
   /* get the arrow type of the plugin */
   arrow_type = xfce_arrow_button_get_arrow_type (XFCE_ARROW_BUTTON (plugin->arrow));
 
-  /* size of the menu items */
-  if (gtk_icon_size_lookup (launcher_menu_icon_size, &w, &h))
-    size = MIN (w, h);
-  else
-    size = MENU_ICON_SIZE;
-
   /* walk through the menu entries */
   for (li = plugin->items, n = 0; li != NULL; li = li->next, n++)
     {
@@ -1571,10 +1565,14 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
 
       /* create the menu item */
       name = garcon_menu_item_get_name (item);
-      mi = gtk_image_menu_item_new_with_label (
-          exo_str_is_empty (name) ? _("Unnamed Item") : name);
+      mi = gtk_menu_item_new ();
+      label = gtk_label_new (panel_str_is_empty (name) ? _("Unnamed Item") : name);
+      gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+      gtk_box_pack_end (GTK_BOX (box), label, TRUE, TRUE, 0);
+      gtk_container_add (GTK_CONTAINER (mi), box);
       g_object_set_qdata (G_OBJECT (mi), launcher_plugin_quark, plugin);
-      gtk_widget_show (mi);
+      gtk_widget_show_all (mi);
       gtk_drag_dest_set (mi, GTK_DEST_DEFAULT_ALL, drop_targets,
                          G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY);
       g_signal_connect (G_OBJECT (mi), "activate",
@@ -1600,11 +1598,10 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
 
       /* set the icon if one is set */
       icon_name = garcon_menu_item_get_icon_name (item);
-      if (!exo_str_is_empty (icon_name))
+      if (!panel_str_is_empty (icon_name))
         {
-          image = xfce_panel_image_new_from_source (icon_name);
-          xfce_panel_image_set_size (XFCE_PANEL_IMAGE (image), size);
-          gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+          image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_DND);
+          gtk_box_pack_start (GTK_BOX (box), image, FALSE, TRUE, 3);
           gtk_widget_show (image);
         }
     }
@@ -1628,8 +1625,6 @@ launcher_plugin_menu_popup (gpointer user_data)
 
   panel_return_val_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin), FALSE);
 
-  GDK_THREADS_ENTER ();
-
   /* construct the menu if needed */
   if (plugin->menu == NULL)
     launcher_plugin_menu_construct (plugin);
@@ -1645,10 +1640,10 @@ launcher_plugin_menu_popup (gpointer user_data)
 
   /* fallback to manual positioning, this is used with
    * drag motion over the arrow button */
-  if (!GTK_WIDGET_VISIBLE (plugin->menu))
+  if (!gtk_widget_get_visible (plugin->menu))
     {
       /* make sure the size is allocated */
-      if (!GTK_WIDGET_REALIZED (plugin->menu))
+      if (!gtk_widget_get_realized (plugin->menu))
         gtk_widget_realize (plugin->menu);
 
       /* use the widget position function to get the coordinates */
@@ -1657,12 +1652,10 @@ launcher_plugin_menu_popup (gpointer user_data)
 
       /* bit ugly... but show the menu */
       gtk_widget_show (plugin->menu);
-      gtk_window_move (GTK_WINDOW (GTK_MENU (plugin->menu)->toplevel), x, y);
-      gtk_widget_show (GTK_MENU (plugin->menu)->toplevel);
+      gtk_window_move (GTK_WINDOW (gtk_widget_get_toplevel (plugin->menu)), x, y);
+      gtk_widget_show (gtk_widget_get_toplevel (plugin->menu));
     }
 
-  GDK_THREADS_LEAVE ();
-
   return FALSE;
 }
 
@@ -1696,6 +1689,7 @@ launcher_plugin_button_update (LauncherPlugin *plugin)
   GarconMenuItem      *item = NULL;
   const gchar         *icon_name;
   XfcePanelPluginMode  mode;
+  gint                 icon_size;
 
   panel_return_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin));
 
@@ -1711,6 +1705,7 @@ launcher_plugin_button_update (LauncherPlugin *plugin)
     item = GARCON_MENU_ITEM (plugin->items->data);
 
   mode = xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin));
+  icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
 
   /* disable the "small" property in the deskbar mode and the label visible */
   if (G_UNLIKELY (plugin->show_label && mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR))
@@ -1729,11 +1724,13 @@ launcher_plugin_button_update (LauncherPlugin *plugin)
     }
   else if (G_LIKELY (item != NULL))
     {
-      panel_return_if_fail (XFCE_IS_PANEL_IMAGE (plugin->child));
+      panel_return_if_fail (GTK_IS_WIDGET (plugin->child));
+
 
       icon_name = garcon_menu_item_get_icon_name (item);
-      xfce_panel_image_set_from_source (XFCE_PANEL_IMAGE (plugin->child),
-          exo_str_is_empty (icon_name) ? GTK_STOCK_MISSING_IMAGE : icon_name);
+      gtk_image_set_from_icon_name (GTK_IMAGE (plugin->child),
+          panel_str_is_empty (icon_name) ? "image-missing" : icon_name,
+          icon_size);
 
       panel_utils_set_atk_info (plugin->button,
           garcon_menu_item_get_name (item),
@@ -1742,9 +1739,9 @@ launcher_plugin_button_update (LauncherPlugin *plugin)
   else
     {
       /* set missing image icon */
-      panel_return_if_fail (XFCE_IS_PANEL_IMAGE (plugin->child));
-      xfce_panel_image_set_from_source (XFCE_PANEL_IMAGE (plugin->child),
-                                        GTK_STOCK_MISSING_IMAGE);
+      panel_return_if_fail (GTK_IS_WIDGET (plugin->child));
+      gtk_image_set_from_icon_name (GTK_IMAGE (plugin->child),
+                                    "image-missing", icon_size);
     }
 }
 
@@ -1755,9 +1752,9 @@ launcher_plugin_button_state_changed (GtkWidget    *button_a,
                                       GtkStateType  state,
                                       GtkWidget    *button_b)
 {
-  if (GTK_WIDGET_STATE (button_a) != GTK_WIDGET_STATE (button_b)
-      && GTK_WIDGET_STATE (button_a) != GTK_STATE_INSENSITIVE)
-    gtk_widget_set_state (button_b, GTK_WIDGET_STATE (button_a));
+  if (gtk_widget_get_state_flags (button_a) != gtk_widget_get_state_flags (button_b)
+      && (gtk_widget_get_state_flags (button_a) & GTK_STATE_INSENSITIVE))
+    gtk_widget_set_state_flags (button_b, gtk_widget_get_state_flags (button_a), TRUE);
 }
 
 
@@ -1792,10 +1789,10 @@ launcher_plugin_button_press_event (GtkWidget      *button,
     {
       /* start the popup timeout */
       plugin->menu_timeout_id =
-          g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
-                              MENU_POPUP_DELAY,
-                              launcher_plugin_menu_popup, plugin,
-                              launcher_plugin_menu_popup_destroyed);
+        gdk_threads_add_timeout_full (G_PRIORITY_DEFAULT_IDLE,
+                                      MENU_POPUP_DELAY,
+                                      launcher_plugin_menu_popup, plugin,
+                                      launcher_plugin_menu_popup_destroyed);
     }
 
   return FALSE;
@@ -1819,7 +1816,6 @@ launcher_plugin_button_release_event (GtkWidget      *button,
 
   /* leave when there are no menu items or there is an internal arrow */
   if (plugin->items == NULL
-      || !GTK_BUTTON (button)->in_button
       || ARROW_INSIDE_BUTTON (plugin))
     return FALSE;
 
@@ -1934,7 +1930,7 @@ launcher_plugin_supported_drop (GdkDragContext *context,
     return GDK_NONE;
 
   /* check if we support the target */
-  for (li = context->targets; li; li = li->next)
+  for (li = gdk_drag_context_list_targets (context); li; li = li->next)
     {
       target = GDK_POINTER_TO_ATOM (li->data);
       for (i = 0; i < G_N_ELEMENTS (drop_targets); i++)
@@ -2022,12 +2018,16 @@ launcher_plugin_button_drag_leave (GtkWidget      *widget,
 
 
 static gboolean
-launcher_plugin_button_expose_event (GtkWidget      *widget,
-                                     GdkEventExpose *event,
-                                     LauncherPlugin *plugin)
+launcher_plugin_button_draw (GtkWidget      *widget,
+                             cairo_t        *cr,
+                             LauncherPlugin *plugin)
 {
-  GtkArrowType arrow_type;
-  gint         size, x, y, thickness, offset;
+  GtkArrowType      arrow_type;
+  gdouble           angle;
+  gint              size, x, y, offset;
+  GtkAllocation     allocation;
+  GtkStyleContext  *ctx;
+  GtkBorder         padding;
 
   panel_return_val_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin), FALSE);
 
@@ -2039,31 +2039,38 @@ launcher_plugin_button_expose_event (GtkWidget      *widget,
   arrow_type = xfce_arrow_button_get_arrow_type (XFCE_ARROW_BUTTON (plugin->arrow));
 
   /* style thickness */
-  thickness = MAX (widget->style->xthickness, widget->style->ythickness);
+  ctx = gtk_widget_get_style_context (widget);
+  gtk_style_context_get_padding (ctx, gtk_widget_get_state_flags (widget), &padding);
 
   /* size of the arrow and the start coordinates */
-  size = widget->allocation.width / 3;
-  x = widget->allocation.x + thickness;
-  y = widget->allocation.y + thickness;
-  offset = size + 2 * thickness;
+  gtk_widget_get_allocation (widget, &allocation);
+
+  size = allocation.width / 3;
+  x = padding.left;
+  y = padding.top;
+  offset = size + padding.left + padding.right;
+  angle = 1.5 * G_PI;
 
   /* calculate the position based on the arrow type */
   switch (arrow_type)
     {
     case GTK_ARROW_UP:
       /* north east */
-      x += widget->allocation.width - offset;
+      x += allocation.width - offset;
+      angle = 0.0 * G_PI;
       break;
 
     case GTK_ARROW_DOWN:
       /* south west */
-      y += widget->allocation.height - offset;
+      y += allocation.height - offset;
+      angle = 1.0 * G_PI;
       break;
 
     case GTK_ARROW_RIGHT:
       /* south east */
-      x += widget->allocation.width - offset;
-      y += widget->allocation.height - offset;
+      x += allocation.width - offset;
+      y += allocation.height - offset;
+      angle = 0.5 * G_PI;
       break;
 
     default:
@@ -2072,10 +2079,7 @@ launcher_plugin_button_expose_event (GtkWidget      *widget,
     }
 
   /* paint the arrow */
-  gtk_paint_arrow (widget->style, widget->window,
-                   GTK_WIDGET_STATE (widget), GTK_SHADOW_IN,
-                   &(event->area), widget, "launcher_button",
-                   arrow_type, TRUE, x, y, size, size);
+  gtk_render_arrow (ctx, cr, angle, (gdouble) x, (gdouble) y, (gdouble) size);
 
   return FALSE;
 }
@@ -2139,9 +2143,9 @@ launcher_plugin_arrow_drag_motion (GtkWidget      *widget,
 
       /* start the popup timeout */
       plugin->menu_timeout_id =
-          g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, MENU_POPUP_DELAY,
-                              launcher_plugin_menu_popup, plugin,
-                              launcher_plugin_menu_popup_destroyed);
+        gdk_threads_add_timeout_full (G_PRIORITY_DEFAULT_IDLE, MENU_POPUP_DELAY,
+                                      launcher_plugin_menu_popup, plugin,
+                                      launcher_plugin_menu_popup_destroyed);
     }
 
   return TRUE;
@@ -2158,7 +2162,7 @@ launcher_plugin_arrow_drag_leave_timeout (gpointer user_data)
   gint            menu_x, menu_y, menu_w, menu_h;
 
   panel_return_val_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin), FALSE);
-  panel_return_val_if_fail (menu == NULL || GDK_IS_WINDOW (menu->window), FALSE);
+  panel_return_val_if_fail (menu == NULL || gtk_widget_get_has_window (menu), FALSE);
 
   /* leave when the menu is destroyed */
   if (G_UNLIKELY (plugin->menu == NULL))
@@ -2169,15 +2173,16 @@ launcher_plugin_arrow_drag_leave_timeout (gpointer user_data)
                            NULL, &pointer_x, &pointer_y, NULL);
 
   /* get the menu position */
-  gdk_window_get_root_origin (menu->window, &menu_x, &menu_y);
-  gdk_drawable_get_size (GDK_DRAWABLE (menu->window), &menu_w, &menu_h);
+  gdk_window_get_root_origin (gtk_widget_get_window (menu), &menu_x, &menu_y);
+  menu_w = gdk_window_get_width (gtk_widget_get_window (menu));
+  menu_h = gdk_window_get_height (gtk_widget_get_window (menu));
 
   /* check if we should hide the menu */
   if (pointer_x < menu_x || pointer_x > menu_x + menu_w
       || pointer_y < menu_y || pointer_y > menu_y + menu_h)
     {
       /* hide the menu */
-      gtk_widget_hide (GTK_MENU (menu)->toplevel);
+      gtk_widget_hide (gtk_widget_get_toplevel (menu));
       gtk_widget_hide (menu);
 
       /* inactive the toggle button */
@@ -2208,7 +2213,7 @@ launcher_plugin_arrow_drag_leave (GtkWidget      *widget,
   else
     {
       /* start a timeout to give the user some time to drag to the menu */
-      g_timeout_add (MENU_POPUP_DELAY, launcher_plugin_arrow_drag_leave_timeout, plugin);
+      gdk_threads_add_timeout (MENU_POPUP_DELAY, launcher_plugin_arrow_drag_leave_timeout, plugin);
     }
 }
 
@@ -2230,11 +2235,11 @@ launcher_plugin_item_query_tooltip (GtkWidget      *widget,
 
   /* require atleast an item name */
   name = garcon_menu_item_get_name (item);
-  if (exo_str_is_empty (name))
+  if (panel_str_is_empty (name))
     return FALSE;
 
   comment = garcon_menu_item_get_comment (item);
-  if (!exo_str_is_empty (comment))
+  if (!panel_str_is_empty (comment))
     {
       markup = g_markup_printf_escaped ("<b>%s</b>\n%s", name, comment);
       gtk_tooltip_set_markup (tooltip, markup);
@@ -2250,7 +2255,7 @@ launcher_plugin_item_query_tooltip (GtkWidget      *widget,
    * data on the menu item widget */
   if (GTK_IS_MENU_ITEM (widget))
     {
-      pixbuf = g_object_get_data (G_OBJECT (widget), I_("pixbuf-cache"));
+      pixbuf = g_object_get_data (G_OBJECT (widget), "pixbuf-cache");
       if (G_LIKELY (pixbuf != NULL))
         {
           gtk_tooltip_set_icon (tooltip, pixbuf);
@@ -2262,7 +2267,7 @@ launcher_plugin_item_query_tooltip (GtkWidget      *widget,
           if (G_LIKELY (pixbuf != NULL))
             {
               gtk_tooltip_set_icon (tooltip, pixbuf);
-              g_object_set_data_full (G_OBJECT (widget), I_("pixbuf-cache"), pixbuf,
+              g_object_set_data_full (G_OBJECT (widget), "pixbuf-cache", pixbuf,
                                       (GDestroyNotify) g_object_unref);
             }
         }
@@ -2330,7 +2335,7 @@ launcher_plugin_item_exec (GarconMenuItem *item,
 
   /* leave when there is nothing to execute */
   command = garcon_menu_item_get_command (item);
-  if (exo_str_is_empty (command))
+  if (panel_str_is_empty (command))
     return;
 
   if (G_UNLIKELY (uri_list != NULL
@@ -2361,8 +2366,8 @@ launcher_plugin_item_exec_from_clipboard (GarconMenuItem *item,
 {
   GtkClipboard     *clipboard;
   gchar            *text = NULL;
-  GSList           *uri_list;
-  GtkSelectionData  data;
+  //GSList           *uri_list;
+  //GtkSelectionData  data;
 
   panel_return_if_fail (GARCON_IS_MENU_ITEM (item));
   panel_return_if_fail (GDK_IS_SCREEN (screen));
@@ -2373,7 +2378,7 @@ launcher_plugin_item_exec_from_clipboard (GarconMenuItem *item,
     text = gtk_clipboard_wait_for_text (clipboard);
 
   /* try the secondary keayboard if the text is empty */
-  if (exo_str_is_empty (text))
+  if (panel_str_is_empty (text))
     {
       /* get the secondary clipboard text */
       clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
@@ -2381,21 +2386,21 @@ launcher_plugin_item_exec_from_clipboard (GarconMenuItem *item,
         text = gtk_clipboard_wait_for_text (clipboard);
     }
 
-  if (!exo_str_is_empty (text))
+  if (!panel_str_is_empty (text))
     {
       /* create fake selection data */
-      data.data = (guchar *) text;
-      data.length = strlen (text);
-      data.target = GDK_NONE;
+      //data.data = (guchar *) text;      //HOWTO?
+      //data.length = strlen (text);
+      //data.target = GDK_NONE;
 
       /* extract the uris from the selection data */
-      uri_list = launcher_plugin_uri_list_extract (&data);
+      //uri_list = launcher_plugin_uri_list_extract (&data);
 
       /* launch with the uri list */
-      launcher_plugin_item_exec (item, event_time,
-                                 screen, uri_list);
+      //launcher_plugin_item_exec (item, event_time,
+      //                           screen, uri_list);
 
-      launcher_plugin_uri_list_free (uri_list);
+      //launcher_plugin_uri_list_free (uri_list);
     }
 
   g_free (text);
@@ -2433,7 +2438,7 @@ launcher_plugin_exec_parse (GarconMenuItem   *item,
 
   /* get the command */
   command = garcon_menu_item_get_command (item);
-  panel_return_val_if_fail (!exo_str_is_empty (command), FALSE);
+  panel_return_val_if_fail (!panel_str_is_empty (command), FALSE);
 
   /* allocate an empty string */
   string = g_string_sized_new (100);
@@ -2481,7 +2486,7 @@ launcher_plugin_exec_parse (GarconMenuItem   *item,
 
             case 'i':
               tmp = garcon_menu_item_get_icon_name (item);
-              if (!exo_str_is_empty (tmp))
+              if (!panel_str_is_empty (tmp))
                 {
                   g_string_append (string, "--icon ");
                   launcher_plugin_exec_append_quoted (string, tmp);
@@ -2490,13 +2495,13 @@ launcher_plugin_exec_parse (GarconMenuItem   *item,
 
             case 'c':
               tmp = garcon_menu_item_get_name (item);
-              if (!exo_str_is_empty (tmp))
+              if (!panel_str_is_empty (tmp))
                 launcher_plugin_exec_append_quoted (string, tmp);
               break;
 
             case 'k':
               uri = garcon_menu_item_get_uri (item);
-              if (!exo_str_is_empty (uri))
+              if (!panel_str_is_empty (uri))
                 launcher_plugin_exec_append_quoted (string, uri);
               g_free (uri);
               break;
@@ -2529,21 +2534,21 @@ launcher_plugin_uri_list_extract (GtkSelectionData *data)
   gchar   *uri;
 
   /* leave if there is no data */
-  if (data->length <= 0)
+  if (gtk_selection_data_get_length (data) <= 0)
     return NULL;
 
   /* extract the files */
-  if (data->target == gdk_atom_intern_static_string ("text/uri-list"))
+  if (gtk_selection_data_get_target (data) == gdk_atom_intern_static_string ("text/uri-list"))
     {
       /* extract the list of uris */
-      array = g_uri_list_extract_uris ((gchar *) data->data);
+      array = g_uri_list_extract_uris ((gchar *) gtk_selection_data_get_data (data));
       if (G_UNLIKELY (array == NULL))
         return NULL;
 
       /* create the list of uris */
       for (i = 0; array[i] != NULL; i++)
         {
-          if (!exo_str_is_empty (array[i]))
+          if (!panel_str_is_empty (array[i]))
             list = g_slist_prepend (list, array[i]);
           else
             g_free (array[i]);
@@ -2554,7 +2559,7 @@ launcher_plugin_uri_list_extract (GtkSelectionData *data)
   else
     {
       /* split the data on new lines */
-      array = g_strsplit_set ((const gchar *) data->data, "\n\r", -1);
+      array = g_strsplit_set ((const gchar *) gtk_selection_data_get_data (data), "\n\r", -1);
       if (G_UNLIKELY (array == NULL))
         return NULL;
 
@@ -2562,14 +2567,14 @@ launcher_plugin_uri_list_extract (GtkSelectionData *data)
       for (i = 0; array[i] != NULL; i++)
         {
           /* skip empty strings */
-          if (!!exo_str_is_empty (array[i]))
+          if (!!panel_str_is_empty (array[i]))
             continue;
 
           uri = NULL;
 
           if (g_path_is_absolute (array[i]))
             uri = g_filename_to_uri (array[i], NULL, NULL);
-          else if (exo_str_looks_like_an_uri (array[i]))
+          else if (_exo_str_looks_like_an_uri (array[i]))
             uri = g_strdup (array[i]);
 
           /* append the uri if we extracted one */
