@@ -27,7 +27,6 @@
 #include <common/panel-xfconf.h>
 #include <common/panel-utils.h>
 #include <common/panel-debug.h>
-#include <exo/exo.h>
 
 #include "systray.h"
 #include "systray-box.h"
@@ -55,8 +54,9 @@ static void     systray_plugin_orientation_changed          (XfcePanelPlugin
 static gboolean systray_plugin_size_changed                 (XfcePanelPlugin       *panel_plugin,
                                                              gint                   size);
 static void     systray_plugin_configure_plugin             (XfcePanelPlugin       *panel_plugin);
-static void     systray_plugin_box_expose_event             (GtkWidget             *box,
-                                                             GdkEventExpose        *event);
+static void     systray_plugin_box_draw                     (GtkWidget             *box,
+                                                             cairo_t               *cr,
+                                                             gpointer               user_data);
 static void     systray_plugin_button_toggled               (GtkWidget             *button,
                                                              SystrayPlugin         *plugin);
 static void     systray_plugin_button_set_arrow             (SystrayPlugin         *plugin);
@@ -182,28 +182,28 @@ systray_plugin_class_init (SystrayPluginClass *klass)
                                                       SIZE_MAX_MIN,
                                                       SIZE_MAX_MAX,
                                                       SIZE_MAX_DEFAULT,
-                                                      EXO_PARAM_READWRITE));
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_FRAME,
                                    g_param_spec_boolean ("show-frame",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_NAMES_HIDDEN,
                                    g_param_spec_boxed ("names-hidden",
                                                        NULL, NULL,
                                                        PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_NAMES_VISIBLE,
                                    g_param_spec_boxed ("names-visible",
                                                        NULL, NULL,
                                                        PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
-                                                       EXO_PARAM_READWRITE));
+                                                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 
@@ -211,6 +211,8 @@ systray_plugin_class_init (SystrayPluginClass *klass)
 static void
 systray_plugin_init (SystrayPlugin *plugin)
 {
+  //GtkRcStyle *style;
+
   plugin->manager = NULL;
   plugin->show_frame = TRUE;
   plugin->idle_startup = 0;
@@ -221,14 +223,20 @@ systray_plugin_init (SystrayPlugin *plugin)
   gtk_frame_set_shadow_type (GTK_FRAME (plugin->frame), GTK_SHADOW_ETCHED_IN);
   gtk_widget_show (plugin->frame);
 
-  plugin->hvbox = xfce_hvbox_new (GTK_ORIENTATION_HORIZONTAL, FALSE, 2);
+  // FIXME
+  //style = gtk_rc_style_new ();
+  //style->xthickness = style->ythickness = 1;
+  //gtk_widget_modify_style (plugin->frame, style);
+  //g_object_unref (G_OBJECT (style));
+
+  plugin->hvbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_container_add (GTK_CONTAINER (plugin->frame), plugin->hvbox);
   gtk_widget_show (plugin->hvbox);
 
   plugin->box = systray_box_new ();
   gtk_box_pack_start (GTK_BOX (plugin->hvbox), plugin->box, TRUE, TRUE, 0);
-  g_signal_connect (G_OBJECT (plugin->box), "expose-event",
-      G_CALLBACK (systray_plugin_box_expose_event), NULL);
+  g_signal_connect (G_OBJECT (plugin->box), "draw",
+      G_CALLBACK (systray_plugin_box_draw), NULL);
   gtk_container_set_border_width (GTK_CONTAINER (plugin->box), FRAME_SPACING);
   gtk_widget_show (plugin->box);
 
@@ -237,7 +245,9 @@ systray_plugin_init (SystrayPlugin *plugin)
   g_signal_connect (G_OBJECT (plugin->button), "toggled",
       G_CALLBACK (systray_plugin_button_toggled), plugin);
   gtk_button_set_relief (GTK_BUTTON (plugin->button), GTK_RELIEF_NONE);
-  exo_binding_new (G_OBJECT (plugin->box), "has-hidden", G_OBJECT (plugin->button), "visible");
+  g_object_bind_property (G_OBJECT (plugin->box), "has-hidden",
+                          G_OBJECT (plugin->button), "visible",
+                          G_BINDING_SYNC_CREATE);
   xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), plugin->button);
 }
 
@@ -298,6 +308,7 @@ systray_plugin_set_property (GObject      *object,
   const GValue  *tmp;
   gchar         *name;
   guint          i;
+  //GtkRcStyle    *style;
 
   switch (prop_id)
     {
@@ -314,8 +325,14 @@ systray_plugin_set_property (GObject      *object,
           gtk_frame_set_shadow_type (GTK_FRAME (plugin->frame),
               show_frame ? GTK_SHADOW_ETCHED_IN : GTK_SHADOW_NONE);
 
-          gtk_container_set_border_width (GTK_CONTAINER (plugin->box),
-              plugin->show_frame ? FRAME_SPACING : 0);
+          // FIXME
+          //style = gtk_rc_style_new ();
+          //style->xthickness = style->ythickness = show_frame ? 1 : 0;
+          //gtk_widget_modify_style (plugin->frame, style);
+          //g_object_unref (G_OBJECT (style));
+
+          systray_plugin_size_changed (XFCE_PANEL_PLUGIN (plugin),
+              xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)));
         }
       break;
 
@@ -361,8 +378,6 @@ systray_plugin_screen_changed_idle (gpointer user_data)
   GdkScreen     *screen;
   GError        *error = NULL;
 
-  GDK_THREADS_ENTER ();
-
   /* create a new manager and register this screen */
   plugin->manager = systray_manager_new ();
   g_signal_connect (G_OBJECT (plugin->manager), "icon-added",
@@ -386,8 +401,6 @@ systray_plugin_screen_changed_idle (gpointer user_data)
       g_error_free (error);
     }
 
-  GDK_THREADS_LEAVE ();
-
   return FALSE;
 }
 
@@ -417,8 +430,8 @@ systray_plugin_screen_changed (GtkWidget *widget,
 
   /* schedule a delayed startup */
   if (plugin->idle_startup == 0)
-    plugin->idle_startup = g_idle_add_full (G_PRIORITY_LOW, systray_plugin_screen_changed_idle,
-                                            plugin, systray_plugin_screen_changed_idle_destroyed);
+    plugin->idle_startup = gdk_threads_add_idle_full (G_PRIORITY_LOW, systray_plugin_screen_changed_idle,
+                                                      plugin, systray_plugin_screen_changed_idle_destroyed);
 }
 
 
@@ -492,7 +505,7 @@ systray_plugin_orientation_changed (XfcePanelPlugin *panel_plugin,
 {
   SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
 
-  xfce_hvbox_set_orientation (XFCE_HVBOX (plugin->hvbox), orientation);
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (plugin->hvbox), orientation);
   systray_box_set_orientation (XFCE_SYSTRAY_BOX (plugin->box), orientation);
 
   if (G_LIKELY (plugin->manager != NULL))
@@ -512,9 +525,11 @@ static gboolean
 systray_plugin_size_changed (XfcePanelPlugin *panel_plugin,
                              gint             size)
 {
-  SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
-  GtkWidget     *frame = plugin->frame;
-  gint           border = 0;
+  SystrayPlugin    *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
+  GtkWidget        *frame = plugin->frame;
+  gint              border = 0;
+  GtkStyleContext  *ctx;
+  GtkBorder         padding;
 
   /* set the frame border */
   if (plugin->show_frame && size > 26)
@@ -525,8 +540,12 @@ systray_plugin_size_changed (XfcePanelPlugin *panel_plugin,
    * behind the allocated size when resizing and during startup, we
    * correct the maximum size set by the user with the size the panel
    * will most likely allocated */
-  border += MAX (frame->style->xthickness, frame->style->ythickness);
-  systray_box_set_size_alloc (XFCE_SYSTRAY_BOX (plugin->box), size - 2 * border);
+
+  ctx = gtk_widget_get_style_context (frame);
+  gtk_style_context_get_padding (ctx, gtk_widget_get_state_flags (frame), &padding);
+
+  border += MAX (padding.left+padding.right, padding.top+padding.bottom);
+  systray_box_set_size_alloc (XFCE_SYSTRAY_BOX (plugin->box), size - border);
 
   return TRUE;
 }
@@ -549,13 +568,15 @@ systray_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 
   object = gtk_builder_get_object (builder, "size-max");
   panel_return_if_fail (GTK_IS_WIDGET (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "size-max",
-                          G_OBJECT (object), "value");
+  g_object_bind_property (G_OBJECT (plugin), "size-max",
+                          G_OBJECT (object), "value",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   object = gtk_builder_get_object (builder, "show-frame");
   panel_return_if_fail (GTK_IS_WIDGET (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "show-frame",
-                          G_OBJECT (object), "active");
+  g_object_bind_property (G_OBJECT (plugin), "show-frame",
+                          G_OBJECT (object), "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   store = gtk_builder_get_object (builder, "applications-store");
   panel_return_if_fail (GTK_IS_LIST_STORE (store));
@@ -579,21 +600,22 @@ systray_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 
 
 static void
-systray_plugin_box_expose_event_icon (GtkWidget *child,
-                                      gpointer   user_data)
+systray_plugin_box_draw_icon (GtkWidget *child,
+                              gpointer   user_data)
 {
   cairo_t       *cr = user_data;
-  GtkAllocation *alloc;
+  GtkAllocation  alloc;
 
   if (systray_socket_is_composited (XFCE_SYSTRAY_SOCKET (child)))
     {
-      alloc = &child->allocation;
+      gtk_widget_get_allocation (child, &alloc);
 
       /* skip hidden (see offscreen in box widget) icons */
-      if (alloc->x > -1 && alloc->y > -1)
+      if (alloc.x > -1 && alloc.y > -1)
         {
-          gdk_cairo_set_source_pixmap (cr, gtk_widget_get_window (child),
-                                       alloc->x, alloc->y);
+          // FIXME
+          gdk_cairo_set_source_window (cr, gtk_widget_get_window (child),
+                                       alloc.x, alloc.y);
           cairo_paint (cr);
         }
     }
@@ -602,27 +624,19 @@ systray_plugin_box_expose_event_icon (GtkWidget *child,
 
 
 static void
-systray_plugin_box_expose_event (GtkWidget      *box,
-                                 GdkEventExpose *event)
+systray_plugin_box_draw (GtkWidget *box,
+                         cairo_t   *cr,
+                         gpointer   user_data)
 {
-  cairo_t *cr;
+  panel_return_if_fail (cr != NULL);
 
   if (!gtk_widget_is_composited (box))
     return;
 
-  cr = gdk_cairo_create (gtk_widget_get_window (box));
-  if (G_LIKELY (cr != NULL))
-    {
-      gdk_cairo_rectangle (cr, &event->area);
-      cairo_clip (cr);
-
-      /* separately draw all the composed tray icons after gtk
-       * handled the expose event */
-      gtk_container_foreach (GTK_CONTAINER (box),
-          systray_plugin_box_expose_event_icon, cr);
-
-      cairo_destroy (cr);
-    }
+  /* separately draw all the composed tray icons after gtk
+   * handled the draw event */
+  gtk_container_foreach (GTK_CONTAINER (box),
+                         (GtkCallback) systray_plugin_box_draw_icon, cr);
 }
 
 
@@ -747,7 +761,7 @@ systray_plugin_names_set_hidden (SystrayPlugin *plugin,
                                  gboolean       hidden)
 {
   panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
-  panel_return_if_fail (!exo_str_is_empty (name));
+  panel_return_if_fail (!panel_str_is_empty (name));
 
   g_hash_table_replace (plugin->names, g_strdup (name),
                         GUINT_TO_POINTER (hidden ? 1 : 0));
@@ -766,7 +780,7 @@ systray_plugin_names_get_hidden (SystrayPlugin *plugin,
 {
   gpointer p;
 
-  if (exo_str_is_empty (name))
+  if (panel_str_is_empty (name))
     return FALSE;
 
   /* lookup the name in the table */
@@ -867,7 +881,7 @@ systray_plugin_dialog_camel_case (const gchar *text)
   gunichar     c;
   GString     *result;
 
-  panel_return_val_if_fail (!exo_str_is_empty (text), NULL);
+  panel_return_val_if_fail (!panel_str_is_empty (text), NULL);
 
   /* allocate a new string for the result */
   result = g_string_sized_new (32);
@@ -919,7 +933,7 @@ systray_plugin_dialog_add_application_names (gpointer key,
   panel_return_if_fail (name == NULL || g_utf8_validate (name, -1, NULL));
 
   /* skip invalid names */
-  if (exo_str_is_empty (name))
+  if (panel_str_is_empty (name))
      return;
 
   /* check if we have a better name for the application */
@@ -1006,7 +1020,7 @@ systray_plugin_dialog_clear_clicked (GtkWidget     *button,
   panel_return_if_fail (XFCE_IS_SYSTRAY_BOX (plugin->box));
 
   if (xfce_dialog_confirm (GTK_WINDOW (gtk_widget_get_toplevel (button)),
-                           GTK_STOCK_CLEAR, NULL, NULL,
+                           "edit-clear", _("Clear"), NULL,
                            _("Are you sure you want to clear the list of "
                              "known applications?")))
     {
