@@ -138,7 +138,6 @@ XFCE_PANEL_DEFINE_PLUGIN_RESIDENT (WindowMenuPlugin, window_menu_plugin)
 
 
 static GQuark window_quark = 0;
-static GtkIconSize menu_icon_size = GTK_ICON_SIZE_INVALID;
 
 
 
@@ -171,35 +170,35 @@ window_menu_plugin_class_init (WindowMenuPluginClass *klass)
                                                       BUTTON_STYLE_ICON,
                                                       BUTTON_STYLE_ARROW,
                                                       BUTTON_STYLE_ICON,
-                                                      EXO_PARAM_READWRITE));
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_WORKSPACE_ACTIONS,
                                    g_param_spec_boolean ("workspace-actions",
                                                          NULL, NULL,
                                                          FALSE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_WORKSPACE_NAMES,
                                    g_param_spec_boolean ("workspace-names",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_URGENTCY_NOTIFICATION,
                                    g_param_spec_boolean ("urgentcy-notification",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ALL_WORKSPACES,
                                    g_param_spec_boolean ("all-workspaces",
                                                          NULL, NULL,
                                                          TRUE,
-                                                         EXO_PARAM_READWRITE));
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_int ("minimized-icon-lucency",
@@ -207,7 +206,7 @@ window_menu_plugin_class_init (WindowMenuPluginClass *klass)
                                                              "Lucent percentage of minimized icons",
                                                              0, 100,
                                                              DEFAULT_ICON_LUCENCY,
-                                                             EXO_PARAM_READABLE));
+                                                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_enum ("ellipsize-mode",
@@ -215,7 +214,7 @@ window_menu_plugin_class_init (WindowMenuPluginClass *klass)
                                                               "The ellipsize mode used for the menu label",
                                                               PANGO_TYPE_ELLIPSIZE_MODE,
                                                               DEFAULT_ELLIPSIZE_MODE,
-                                                              EXO_PARAM_READABLE));
+                                                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_int ("max-width-chars",
@@ -223,13 +222,9 @@ window_menu_plugin_class_init (WindowMenuPluginClass *klass)
                                                              "Maximum length of window/workspace name",
                                                              1, G_MAXINT,
                                                              DEFAULT_MAX_WIDTH_CHARS,
-                                                             EXO_PARAM_READABLE));
+                                                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   window_quark = g_quark_from_static_string ("window-list-window-quark");
-
-  menu_icon_size = gtk_icon_size_from_name ("panel-window-menu");
-  if (menu_icon_size == GTK_ICON_SIZE_INVALID)
-    menu_icon_size = gtk_icon_size_register ("panel-window-menu", 16, 16);
 }
 
 
@@ -256,7 +251,7 @@ window_menu_plugin_init (WindowMenuPlugin *plugin)
   g_signal_connect (G_OBJECT (plugin->button), "toggled",
       G_CALLBACK (window_menu_plugin_menu), plugin);
 
-  plugin->icon = xfce_panel_image_new_from_source ("user-desktop");
+  plugin->icon = gtk_image_new_from_icon_name ("user-desktop", GTK_ICON_SIZE_BUTTON);
   gtk_container_add (GTK_CONTAINER (plugin->button), plugin->icon);
   gtk_widget_show (plugin->icon);
 }
@@ -518,6 +513,7 @@ window_menu_plugin_size_changed (XfcePanelPlugin *panel_plugin,
                                  gint             size)
 {
   WindowMenuPlugin *plugin = XFCE_WINDOW_MENU_PLUGIN (panel_plugin);
+  gint              button_size;
 
   if (plugin->button_style == BUTTON_STYLE_ICON)
     {
@@ -530,12 +526,19 @@ window_menu_plugin_size_changed (XfcePanelPlugin *panel_plugin,
       /* set the size of the arrow button */
       if (xfce_panel_plugin_get_orientation (panel_plugin) ==
               GTK_ORIENTATION_HORIZONTAL)
-        gtk_widget_set_size_request (GTK_WIDGET (plugin),
-                                     ARROW_BUTTON_SIZE, -1);
+        {
+          gtk_widget_get_preferred_width (plugin->button, NULL, &button_size);
+          gtk_widget_set_size_request (GTK_WIDGET (plugin), button_size, -1);
+        }
       else
-        gtk_widget_set_size_request (GTK_WIDGET (plugin),
-                                     -1, ARROW_BUTTON_SIZE);
+        {
+          gtk_widget_get_preferred_height (plugin->button, NULL, &button_size);
+          gtk_widget_set_size_request (GTK_WIDGET (plugin), -1, button_size);
+        }
     }
+  /* Update the plugin's pixbuf size too */
+  if (plugin->screen != NULL)
+    window_menu_plugin_active_window_changed (plugin->screen, NULL, plugin);
 
   return TRUE;
 }
@@ -565,8 +568,9 @@ window_menu_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
     {
       object = gtk_builder_get_object (builder, names[i]);
       panel_return_if_fail (GTK_IS_WIDGET (object));
-      exo_mutual_binding_new (G_OBJECT (plugin), names[i],
-                              G_OBJECT (object), "active");
+      g_object_bind_property (G_OBJECT (plugin), names[i],
+                              G_OBJECT (object), "active",
+                              G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
     }
 
   gtk_widget_show (GTK_WIDGET (dialog));
@@ -584,7 +588,7 @@ window_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
   panel_return_val_if_fail (value == NULL || G_IS_VALUE (value), FALSE);
 
   if (strcmp (name, "popup") == 0
-      && GTK_WIDGET_VISIBLE (panel_plugin)
+      && gtk_widget_get_visible (GTK_WIDGET (panel_plugin))
       && !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (plugin->button))
       && panel_utils_grab_available ())
     {
@@ -617,14 +621,16 @@ window_menu_plugin_active_window_changed (WnckScreen       *screen,
 {
   WnckWindow     *window;
   GdkPixbuf      *pixbuf;
-  XfcePanelImage *icon = XFCE_PANEL_IMAGE (plugin->icon);
+  gint            icon_size;
+  GtkWidget      *icon = GTK_WIDGET (plugin->icon);
   WnckWindowType  type;
 
   panel_return_if_fail (XFCE_IS_WINDOW_MENU_PLUGIN (plugin));
-  panel_return_if_fail (XFCE_IS_PANEL_IMAGE (icon));
+  panel_return_if_fail (GTK_IMAGE (icon));
   panel_return_if_fail (WNCK_IS_SCREEN (screen));
   panel_return_if_fail (plugin->screen == screen);
 
+  icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
   /* only do this when the icon is visible */
   if (plugin->button_style == BUTTON_STYLE_ICON)
     {
@@ -640,18 +646,25 @@ window_menu_plugin_active_window_changed (WnckScreen       *screen,
           gtk_widget_set_tooltip_text (GTK_WIDGET (icon),
                                        wnck_window_get_name (window));
 
-          pixbuf = wnck_window_get_icon (window);
-          if (G_LIKELY (pixbuf != NULL))
-            xfce_panel_image_set_from_pixbuf (icon, pixbuf);
+          if (icon_size <= 31)
+            pixbuf = wnck_window_get_mini_icon (window);
           else
-            xfce_panel_image_set_from_source (icon, GTK_STOCK_MISSING_IMAGE);
+            pixbuf = wnck_window_get_icon (window);
+
+          if (G_LIKELY (pixbuf != NULL))
+            gtk_image_set_from_pixbuf (GTK_IMAGE (icon), pixbuf);
+          else {
+            gtk_image_set_from_icon_name (GTK_IMAGE (icon), "image-missing", icon_size);
+            gtk_image_set_pixel_size (GTK_IMAGE (icon), icon_size);
+          }
         }
       else
         {
           show_desktop_icon:
 
           /* desktop is shown right now */
-          xfce_panel_image_set_from_source (icon, "user-desktop");
+          gtk_image_set_from_icon_name (GTK_IMAGE (icon), "user-desktop", icon_size);
+          gtk_image_set_pixel_size (GTK_IMAGE (icon), icon_size);
           gtk_widget_set_tooltip_text (GTK_WIDGET (icon), _("Desktop"));
         }
     }
@@ -849,11 +862,11 @@ window_menu_plugin_menu_workspace_item_new (WnckWorkspace        *workspace,
 
   /* try to get an utf-8 valid name */
   name = wnck_workspace_get_name (workspace);
-  if (!exo_str_is_empty (name)
+  if (!panel_str_is_empty (name)
       && !g_utf8_validate (name, -1, NULL))
     name = utf8 = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
 
-  if (exo_str_is_empty (name))
+  if (panel_str_is_empty (name))
     name = name_num = g_strdup_printf (_("Workspace %d"),
         wnck_workspace_get_number (workspace) + 1);
 
@@ -904,7 +917,7 @@ window_menu_plugin_menu_window_item_activate (GtkWidget      *mi,
 
   panel_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
   panel_return_val_if_fail (GTK_IS_MENU_ITEM (mi), FALSE);
-  panel_return_val_if_fail (GTK_IS_MENU_SHELL (mi->parent), FALSE);
+  panel_return_val_if_fail (GTK_IS_MENU_SHELL (gtk_widget_get_parent (mi)), FALSE);
 
   /* only respond to a button releases */
   if (event->type != GDK_BUTTON_RELEASE)
@@ -959,10 +972,10 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
 
   /* try to get an utf-8 valid name */
   name = wnck_window_get_name (window);
-  if (!exo_str_is_empty (name) && !g_utf8_validate (name, -1, NULL))
+  if (!panel_str_is_empty (name) && !g_utf8_validate (name, -1, NULL))
     name = utf8 = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
 
-  if (exo_str_is_empty (name))
+  if (panel_str_is_empty (name))
     name = "?";
 
   /* store the tooltip text */
@@ -975,7 +988,9 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
     name = decorated = g_strdup_printf ("[%s]", name);
 
   /* create the menu item */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   mi = gtk_image_menu_item_new_with_label (name);
+G_GNUC_END_IGNORE_DEPRECATIONS
   gtk_widget_set_tooltip_text (mi, tooltip);
   g_object_set_qdata (G_OBJECT (mi), window_quark, window);
   g_signal_connect (G_OBJECT (mi), "button-release-event",
@@ -1007,7 +1022,7 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
 
       if (pixbuf != NULL)
         {
-          /* scle the icon if needed */
+          /* scale the icon if needed */
           if (gdk_pixbuf_get_width (pixbuf) > icon_w
               || gdk_pixbuf_get_height (pixbuf) > icon_h)
             {
@@ -1020,14 +1035,18 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
           if (wnck_window_is_minimized (window)
               && plugin->minimized_icon_lucency < 100)
             {
+#ifdef EXO_CHECK_VERSION
               lucent = exo_gdk_pixbuf_lucent (pixbuf, plugin->minimized_icon_lucency);
               if (G_LIKELY (lucent != NULL))
                 pixbuf = lucent;
+#endif
             }
 
           /* set the menu item label */
           image = gtk_image_new_from_pixbuf (pixbuf);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
           gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
           gtk_widget_show (image);
 
           if (lucent != NULL)
@@ -1053,7 +1072,7 @@ window_menu_plugin_menu_selection_done (GtkWidget *menu,
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
 
   /* delay destruction so we can handle the activate event first */
-  exo_gtk_object_destroy_later (GTK_OBJECT (menu));
+  panel_utils_destroy_later (GTK_WIDGET (menu));
 }
 
 
@@ -1072,15 +1091,15 @@ window_menu_plugin_menu_key_press_event (GtkWidget   *menu,
   /* construct an event */
   switch (event->keyval)
     {
-    case GDK_space:
-    case GDK_Return:
-    case GDK_KP_Space:
-    case GDK_KP_Enter:
+    case GDK_KEY_space:
+    case GDK_KEY_Return:
+    case GDK_KEY_KP_Space:
+    case GDK_KEY_KP_Enter:
       /* active the menu item */
       fake_event.button = 1;
       break;
 
-    case GDK_Menu:
+    case GDK_KEY_Menu:
       /* popup the window actions menu */
       fake_event.button = 3;
       break;
@@ -1149,8 +1168,7 @@ window_menu_plugin_menu_new (WindowMenuPlugin *plugin)
   italic = pango_font_description_from_string ("italic");
   bold = pango_font_description_from_string ("bold");
 
-  if (!gtk_icon_size_lookup (menu_icon_size, &w, &h))
-    w = h = 16;
+  w = h = GTK_ICON_SIZE_MENU;
 
   menu = gtk_menu_new ();
   g_signal_connect (G_OBJECT (menu), "key-press-event",
@@ -1300,31 +1318,37 @@ window_menu_plugin_menu_new (WindowMenuPlugin *plugin)
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_show (mi);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       mi = gtk_image_menu_item_new_with_label (_("Add Workspace"));
+G_GNUC_END_IGNORE_DEPRECATIONS
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       g_signal_connect (G_OBJECT (mi), "activate",
           G_CALLBACK (window_menu_plugin_workspace_add), plugin);
       gtk_widget_show (mi);
 
-      image = gtk_image_new_from_stock (GTK_STOCK_ADD, menu_icon_size);
+      image = gtk_image_new_from_icon_name ("list-add", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
       gtk_widget_show (mi);
 
       if (G_LIKELY (workspace != NULL))
         {
           /* try to get an utf-8 valid name */
           name = wnck_workspace_get_name (workspace);
-          if (!exo_str_is_empty (name) && !g_utf8_validate (name, -1, NULL))
+          if (!panel_str_is_empty (name) && !g_utf8_validate (name, -1, NULL))
             name = utf8 = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
         }
 
       /* create label */
-      if (!exo_str_is_empty (name))
+      if (!panel_str_is_empty (name))
         label = g_strdup_printf (_("Remove Workspace \"%s\""), name);
       else
         label = g_strdup_printf (_("Remove Workspace %d"), n_workspaces);
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       mi = gtk_image_menu_item_new_with_label (label);
+G_GNUC_END_IGNORE_DEPRECATIONS
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
       gtk_widget_set_sensitive (mi, !!(n_workspaces > 1));
       g_signal_connect (G_OBJECT (mi), "activate",
@@ -1334,8 +1358,10 @@ window_menu_plugin_menu_new (WindowMenuPlugin *plugin)
       g_free (label);
       g_free (utf8);
 
-      image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, menu_icon_size);
+      image = gtk_image_new_from_icon_name ("list-remove", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
       gtk_widget_show (mi);
     }
 
